

#include <QApplication>
#include <QDesktopWidget>
#include <QFileInfo>
#include <QPushButton>
#include <QProcess>
#include <QThread>
#include <QTextStream>
#include <QGraphicsDropShadowEffect>

#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
#include <unistd.h>
#endif

#include "mainwindow.h"


#define DEFAULT_BACKGROUND_IMAGE_PATH "/usr/share/design/current/backgrounds/default.png"

#define ITEM_DATA_LEVEL_IDX 1001
#define ITEM_DATA_CATEGORY_IDX 1002

#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
#FIXME uncompatable typedef QRegExp QRegularExpression;
#endif

extern const char *__progname;

MainWindow::MainWindow(QWidget *parent)
    : QWidget(parent)
{
    m_level_min = m_level_max = m_level_cur = 0;
#ifdef ENABLE_HOLDER
    m_key_path = QString(QLatin1String("%1/secures/se_app%2")).arg(QLatin1String(qgetenv("TMPDIR"))).arg(QLatin1String(qgetenv("DISPLAY")));
    m_shmem = new QSharedMemory(this);
    m_shmem->setNativeKey(m_key_path);
#endif

    QPalette this_palette(palette());

    QRect screen_geometry(QApplication::desktop()->screenGeometry());
    setGeometry(screen_geometry);

    QFileInfo bg_file_info(QLatin1String(DEFAULT_BACKGROUND_IMAGE_PATH));
    if( bg_file_info.exists()
        && bg_file_info.isFile() ) {
        QPixmap bg_pixmap(QLatin1String(DEFAULT_BACKGROUND_IMAGE_PATH));
        if( !bg_pixmap.isNull() ) {
            if( screen_geometry.size() != bg_pixmap.size() ) {
                bg_pixmap = bg_pixmap.scaled(screen_geometry.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
            }
            QPalette pal(this_palette);
            pal.setBrush(QPalette::Background,QBrush(bg_pixmap));
            setPalette(pal);
        }
    }

    QGraphicsDropShadowEffect *text_shadow = new QGraphicsDropShadowEffect(this);
    text_shadow->setColor(this_palette.color(QPalette::Base));
    text_shadow->setBlurRadius(5);
    text_shadow->setOffset(0, 0);

    w_info = new QLabel(tr("Select level and categories."), this);
    w_info->setAlignment(Qt::AlignCenter);
    w_info->setGraphicsEffect(text_shadow);
    w_levels = new QListWidget(this);
    w_levels->setSelectionMode(QAbstractItemView::SingleSelection);
    w_categories = new QListWidget(this);
    w_categories->setSelectionMode(QAbstractItemView::MultiSelection);
    w_current_range = new QLabel( this);
    w_current_range->setAlignment(Qt::AlignCenter);
    w_current_range->setGraphicsEffect(text_shadow);
    w_buttonbox = new QDialogButtonBox(this);
    w_buttonbox->setCenterButtons(true);
    QPushButton *btn = w_buttonbox->addButton(QDialogButtonBox::Apply);
    w_message = new QLabel( this);
    w_message->setAlignment(Qt::AlignCenter);
    w_message->setGraphicsEffect(text_shadow);
    w_message->hide();

    l_vlayout1 = new QVBoxLayout(this);
    l_vlayout1->addStretch(1);
    l_vlayout1->addWidget(w_info);

    l_hlayout1 = new QHBoxLayout();
    l_vlayout1->addLayout(l_hlayout1);

    l_hlayout1->addStretch(1);
    l_hlayout1->addWidget(w_levels);
    l_hlayout1->addWidget(w_categories);
    l_hlayout1->addStretch(1);

    l_vlayout1->addWidget(w_current_range);
    l_vlayout1->addWidget(w_buttonbox, Qt::AlignCenter);
    l_vlayout1->addWidget(w_message);
    l_vlayout1->addStretch(1);

    setTabOrder(w_levels, w_categories);
    setTabOrder(w_categories, btn);

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
    connect(btn, &QPushButton::clicked, this, &MainWindow::buttonPressed);
    connect(w_levels, &QListWidget::currentItemChanged, this, &MainWindow::levelClicked);
    connect(w_categories, &QListWidget::itemSelectionChanged, this, &MainWindow::categoriesSelected);
#else
    connect(btn, SIGNAL(clicked(bool)), this, SLOT(buttonPressed(bool)));
    connect(w_levels, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), this, SLOT(levelClicked(QListWidgetItem*, QListWidgetItem*)));
    connect(w_categories, SIGNAL(itemSelectionChanged()), this, SLOT(categoriesSelected()));
#endif
}

MainWindow::~MainWindow()
{
}

void MainWindow::setRange(const QString &str_urange)
{
    QStringList lst_urange(str_urange.split(QLatin1Char('-'), QString::SkipEmptyParts));
    if( lst_urange.size() < 2 ) {
	qWarning("%s: unable split initial range", __progname);
    }
    QString urange1;
    if( lst_urange.size() > 0 )
	urange1 = lst_urange.at(0);
    QStringList lst_urange1(urange1.split(QLatin1Char(':'), QString::SkipEmptyParts));
    if( lst_urange1.size() < 2 ) {
	qWarning("%s: unable split range1", __progname);
    }
    QString urange2(lst_urange.at(1));
    QStringList lst_urange2(urange2.split(QLatin1Char(':'), QString::SkipEmptyParts));
    if( lst_urange2.size() < 2 ) {
	qWarning("%s: unable split range2", __progname);
    }
    bool ok;
    QString str_level_min(QLatin1String("s0"));
    if( lst_urange1.size() > 0 )
	str_level_min = lst_urange1.at(0);
    m_level_min = str_level_min.remove(QRegularExpression(QLatin1String("[^\\d]"))).toInt(&ok);
    if( !ok ) {
	m_level_min = 0;
	qWarning("%s: unable convert minimum level", __progname);
    }
    QString str_level_max(QLatin1String("s0"));
    if( lst_urange2.size() > 0 )
	str_level_max = lst_urange2.at(0);
    m_level_max = str_level_max.remove(QRegularExpression(QLatin1String("[^\\d]"))).toInt(&ok);
    if( !ok ) {
	m_level_max = 0;
	qWarning("%s: unable convert maximum level", __progname);
    }

    m_categories.clear();
    QString str_categories_avail;
    if( lst_urange2.size() > 1 )
	str_categories_avail = lst_urange2.at(1);
    Q_FOREACH(QString str_category_base, str_categories_avail.split(QLatin1Char(','), QString::SkipEmptyParts)) {
	QStringList lst_category_base(str_category_base.split(QLatin1Char('.'), QString::SkipEmptyParts));
	int category_min = 0;
	int category_max = 0;
	bool ok;
	category_min = QString(lst_category_base.at(0)).remove(QRegularExpression(QLatin1String("[^\\d]"))).toInt(&ok);
	if( !ok ) {
	    category_min = 0;
	    qWarning("%s: unable convert minimum category", __progname);
	}
	category_max = category_min;
	if( lst_category_base.size() > 1 ) {
	    if( lst_category_base.size() > 1 ) {
		category_max = QString(lst_category_base.at(1)).remove(QRegularExpression(QLatin1String("[^\\d]"))).toInt(&ok);
		if( !ok ) {
		    category_max = category_min;
		    qWarning("%s: unable convert maximum category", __progname);
		}
	    }
	}
	for( int i = category_min; i <= category_max; i++ ) {
	    //qDebug("%s: category %d", __progname, i);
	    m_categories[i] = 0;
	}
    }
    QString str_categories_cur;
    if( lst_urange1.size() > 1 )
	str_categories_cur = lst_urange1.at(1);
    Q_FOREACH(QString str_category_base, str_categories_cur.split(QLatin1Char(','), QString::SkipEmptyParts)) {
	QStringList lst_category_base(str_category_base.split(QLatin1Char('.'), QString::SkipEmptyParts));
	int category_min = 0;
	int category_max = 0;
	bool ok;
	category_min = QString(lst_category_base.at(0)).remove(QRegularExpression(QLatin1String("[^\\d]"))).toInt(&ok);
	if( !ok ) {
	    category_min = 0;
	    qWarning("%s: unable convert minimum current category", __progname);
	}
	category_max = category_min;
	if( lst_category_base.size() > 1 ) {
	    if( lst_category_base.size() > 1 ) {
		category_max = QString(lst_category_base.at(1)).remove(QRegularExpression(QLatin1String("[^\\d]"))).toInt(&ok);
		if( !ok ) {
		    category_max = category_min;
		    qWarning("%s: unable convert maximum current category", __progname);
		}
	    }
	}
	for( int i = category_min; i <= category_max; i++ ) {
	    if( m_categories.contains(i) )
		m_categories[i] = 1;
	}
    }
    applyGuiRange();
}

void MainWindow::applyGuiRange() {
    w_levels->blockSignals(true);
    w_categories->blockSignals(true);

    w_levels->clear();
    w_categories->clear();

    for( int i = m_level_min; i <= m_level_max; i++ ) {
	QString str_level_id(QString(QLatin1String("s%1")).arg(i));
	QString str_level_name(m_setrans.tr(str_level_id).tr);
	QListWidgetItem *item = new QListWidgetItem(str_level_name, w_levels);
	item->setData(ITEM_DATA_LEVEL_IDX, QVariant(i));
    }

    { // set default level
	QListWidgetItem *first = w_levels->item(0);
	if( first ) {
	    m_level_cur = first->data(ITEM_DATA_LEVEL_IDX).toInt();
	    first->setSelected(true);
	}
    }

    SCategories::const_iterator it;
    for(it = m_categories.constBegin(); it != m_categories.constEnd(); ++it )
    {
	QString str_category_id(QString(QLatin1String("c%1")).arg(it.key()));
	QString str_category_name(m_setrans.tr(str_category_id).tr);
	QListWidgetItem *item = new QListWidgetItem(str_category_name, w_categories);
	item->setData(ITEM_DATA_CATEGORY_IDX, QVariant(it.key()));
	item->setSelected(it.value() > 0);
    }

    w_levels->blockSignals(false);
    w_categories->blockSignals(false);
}

void MainWindow::levelClicked(QListWidgetItem *current_new, QListWidgetItem*)
{
    if( current_new )
    {
	m_level_cur = current_new->data(ITEM_DATA_LEVEL_IDX).toInt();
	//qDebug("%s: current level<%d>", __progname, m_level_cur);
    }
}

void MainWindow::categoriesSelected()
{
    SCategories::iterator it;
    for(it = m_categories.begin(); it != m_categories.end(); ++it ) {
	it.value() = 0;
    }
    Q_FOREACH(QListWidgetItem *item, w_categories->selectedItems()) {
	m_categories[item->data(ITEM_DATA_CATEGORY_IDX).toInt()] = 1;
    }
}

QString MainWindow::range()
{
    QString user_range;
    user_range.append(QString(QLatin1String("s%1")).arg(m_level_cur));
    if( m_categories.key(1, -1) >= 0 ) {
	user_range.append(QLatin1Char(':'));
	QStringList str_categories;
	SCategories::const_iterator it(m_categories.constBegin());
	while (it != m_categories.constEnd()) {
	    if( it.value() == 1 ) {
		str_categories.append(QString(QLatin1String("c%1")).arg(it.key()));
	    }
	    ++it;
	}
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
	user_range.append(str_categories.join(QLatin1Char(',')));
#else
	user_range.append(str_categories.join(QLatin1String(",")));
#endif
    }
    return user_range;
}

void MainWindow::buttonPressed(bool)
{
    apply();
}

void MainWindow::apply()
{
    QByteArray user_range;
    user_range.push_back((char)m_level_cur);
    Q_FOREACH( int category_enabled, m_categories.values() ) {
	user_range.push_back((char)category_enabled);
    }

#ifdef ENABLE_HOLDER
    if( m_shmem->size() <= 0 ) {
	if( !m_shmem->create(user_range.size(), QSharedMemory::ReadWrite) ) {
	    switch( m_shmem->error() ) {
		case QSharedMemory::NoError:
		case QSharedMemory::AlreadyExists:
		    break;
		default:
		    qWarning("%s: shared memory creation error<%d>: %s", __progname, m_shmem->error(), qPrintable(m_shmem->errorString()));
		    break;
	    }
	}
	if( !m_shmem->attach(QSharedMemory::ReadWrite) ) {
	    switch( m_shmem->error() ) {
		case QSharedMemory::NoError:
		    break;
		default:
		    qDebug("%s: shared memory attach error<%d>: <%s>", __progname, m_shmem->error(), qPrintable(m_shmem->errorString()));
		    break;
	    }
	}
    }

    if( m_shmem->isAttached() ) {
	memcpy(m_shmem->data(), user_range.data(), user_range.size());
    } else {
	qWarning("%s: shared memory is not attached", __progname);
    }

    if( QProcess::startDetached(QLatin1String("sechooser-holder")) ) {
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
	QThread::msleep(300);
#else
	::sleep(1);
#endif
    } else {
	qWarning("%s: unable to start holder", __progname);
    }
#endif //ENABLE_HOLDER

    QString str_range(range());
    if( !str_range.isEmpty() ) {
#ifdef DONT_IGNORE_LOWEST_LEVEL
	if( true ) {
#else
	if( str_range != QStringLiteral("s0-s0") && str_range != QStringLiteral("s0") ) {
#endif
	    QTextStream(stdout) << str_range << "\n";
	}
    }

    QCoreApplication::exit(0);
}
